/*	$NetBSD: netinet_component.c,v 1.13 2022/09/03 02:48:01 thorpej Exp $	*/

/*
 * Copyright (c) 2009 Antti Kantee.  All Rights Reserved.
 *
 * Development of this software was supported by The Nokia Foundation
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__KERNEL_RCSID(0, "$NetBSD: netinet_component.c,v 1.13 2022/09/03 02:48:01 thorpej Exp $");

#include <sys/param.h>
#include <sys/domain.h>
#include <sys/protosw.h>
#include <sys/socketvar.h>

#include <net/if.h>
#include <netinet/in.h>
#include <netinet/in_var.h>
#include <netinet/ip_var.h>
#include <netinet/if_inarp.h>

#include <rump-sys/kern.h>
#include <rump-sys/net.h>

RUMP_COMPONENT(RUMP_COMPONENT_NET)
{
	extern struct domain arpdomain, inetdomain;

	domain_attach(&arpdomain);
	domain_attach(&inetdomain);
}

int carpattach(int);

RUMP_COMPONENT(RUMP_COMPONENT_NET_IF)
{

	carpattach(1);
}

RUMP_COMPONENT(RUMP_COMPONENT_NET_IFCFG)
{
	struct ifaliasreq ia;
	struct sockaddr_in *sin;
	struct socket *so;
	int error;

	if (lo0ifp == NULL)
		panic("lo0 config: rumpnet_net has not been initialized");

	if ((error = socreate(AF_INET, &so, SOCK_DGRAM, 0, curlwp, NULL)) != 0)
		panic("lo0 config: cannot create socket");

	/* configure 127.0.0.1 for lo0 */
	memset(&ia, 0, sizeof(ia));
	strcpy(ia.ifra_name, "lo0");
	sin = (struct sockaddr_in *)&ia.ifra_addr;
	sin->sin_family = AF_INET;
	sin->sin_len = sizeof(struct sockaddr_in);
	sin->sin_addr.s_addr = inet_addr("127.0.0.1");

	sin = (struct sockaddr_in *)&ia.ifra_mask;
	sin->sin_family = AF_INET;
	sin->sin_len = sizeof(struct sockaddr_in);
	sin->sin_addr.s_addr = inet_addr("255.0.0.0");

	sin = (struct sockaddr_in *)&ia.ifra_broadaddr;
	sin->sin_family = AF_INET;
	sin->sin_len = sizeof(struct sockaddr_in);
	sin->sin_addr.s_addr = inet_addr("127.255.255.255");

	KERNEL_LOCK(1, NULL);
	IFNET_LOCK(lo0ifp);
	in_control(so, SIOCAIFADDR, &ia, lo0ifp);
	IFNET_UNLOCK(lo0ifp);
	KERNEL_UNLOCK_ONE(NULL);
	if_up(lo0ifp);
	soclose(so);
}
